@@ -14,8 +14,8 @@
  * Relations:   SCCP Device -> SCCP DeviceLine -> SCCP Line
  *              SCCP Line -> SCCP ButtonConfig -> SCCP Device
  *
- * \date        $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * \version     $Revision: 5790 $
+ * \date        $Date: 2014-11-20 15:57:48 +0100 (Thu, 20 Nov 2014) $
+ * \version     $Revision: 5803 $
  */
 
 #include <config.h>
@@ -31,7 +31,7 @@
 #include "sccp_indicate.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5803 $")
 int __sccp_device_destroy(const void *ptr);
 void sccp_device_removeFromGlobals(sccp_device_t * device);
 int sccp_device_destroy(const void *ptr);
@@ -934,9 +934,6 @@ int sccp_dev_send(const sccp_device_t * d, sccp_msg_t * msg)
 
 	if (d && d->session && msg) {
 		sccp_log((DEBUGCAT_MESSAGE)) (VERBOSE_PREFIX_3 "%s: >> Send message %s\n", d->id, msgtype2str(letohl(msg->header.lel_messageId)));
-		if (msg && (GLOB(debug) & (DEBUGCAT_MESSAGE)) != 0) {
-			sccp_dump_msg(msg);
-		}
 		result = sccp_session_send(d, msg);
 	} else {
 		sccp_free(msg);
@@ -1085,7 +1082,7 @@ void sccp_dev_set_keyset(const sccp_device_t * d, uint8_t lineInstance, uint32_t
 	//msg->data.SelectSoftKeysMessage.les_validKeyMask = 0xFFFFFFFF;           /* htolel(65535); */
 	msg->data.SelectSoftKeysMessage.les_validKeyMask = htolel(d->softKeyConfiguration.activeMask[softKeySetIndex]);
 
-	sccp_log((DEBUGCAT_SOFTKEY + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Send softkeyset to %s(%d) on line %d  and call %d\n", d->id, skinny_keymode2str(softKeySetIndex), softKeySetIndex, lineInstance, callid);
+	sccp_log((DEBUGCAT_SOFTKEY + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Set softkeyset to %s(%d) on line %d  and call %d\n", d->id, skinny_keymode2str(softKeySetIndex), softKeySetIndex, lineInstance, callid);
 	sccp_log((DEBUGCAT_SOFTKEY + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: validKeyMask %u\n", d->id, msg->data.SelectSoftKeysMessage.les_validKeyMask);
 	sccp_dev_send(d, msg);
 }
