@@ -1,15 +1,15 @@
 /*!
- * \file	sccp_device.h
+ * \file        sccp_device.h
  * \brief       SCCP Device Header
- * \author	Sergio Chersovani <mlists [at] c-net.it>
- * \note	Reworked, but based on chan_sccp code.
- *		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- *		Modified by Jan Czmok and Julien Goodwin
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
+ * \author      Sergio Chersovani <mlists [at] c-net.it>
+ * \note        Reworked, but based on chan_sccp code.
+ *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ *              Modified by Jan Czmok and Julien Goodwin
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-07-10 15:55:26 +0200 (Thu, 10 Jul 2014) $
- * $Revision: 5598 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 
 #ifndef __SCCP_DEVICE_H
@@ -28,8 +28,8 @@ struct sccp_device_indication_cb {
 	void (*const remoteOnhook) (const sccp_device_t * device, sccp_linedevices_t * linedevice, const sccp_channel_t * channel);
 	void (*const offhook) (const sccp_device_t * device, sccp_linedevices_t * linedevice, uint32_t callid);
 	void (*const onhook) (const sccp_device_t * device, const uint8_t lineInstance, uint32_t callid);
-	void (*const dialing) (const sccp_device_t *device, const uint8_t lineInstance, const sccp_channel_t *channel);
-	void (*const proceed) (const sccp_device_t *device, const uint8_t lineInstance, const sccp_channel_t *channel);
+	void (*const dialing) (const sccp_device_t * device, const uint8_t lineInstance, const sccp_channel_t * channel);
+	void (*const proceed) (const sccp_device_t * device, const uint8_t lineInstance, const sccp_channel_t * channel);
 	void (*const connected) (const sccp_device_t * device, sccp_linedevices_t * linedevice, const sccp_channel_t * channel);
 };
 
@@ -88,7 +88,7 @@ void sccp_dev_clear_message(sccp_device_t * d, const boolean_t cleardb);
 void sccp_device_addMessageToStack(sccp_device_t * device, const uint8_t priority, const char *message);
 void sccp_device_clearMessageFromStack(sccp_device_t * device, const uint8_t priority);
 void sccp_device_featureChangedDisplay(const sccp_event_t * event);
-void sccp_device_sendcallstate(const sccp_device_t * d, uint8_t instance, uint32_t callid, uint8_t state, skinny_callpriority_t priority, skinny_callinfo_visibility_t visibility);
+void sccp_device_sendcallstate(const sccp_device_t * d, uint8_t instance, uint32_t callid, skinny_callstate_t state, skinny_callpriority_t priority, skinny_callinfo_visibility_t visibility);
 
 int sccp_dev_send(const sccp_device_t * d, sccp_msg_t * msg);
 int sccp_device_check_ringback(sccp_device_t * d);
@@ -102,6 +102,7 @@ boolean_t sccp_device_check_update(sccp_device_t * d);
 
 // find device
 sccp_device_t *sccp_device_find_byid(const char *id, boolean_t useRealtime);
+
 #if DEBUG
 #define sccp_device_find_byname(_x) __sccp_device_find_byid(_x)
 #ifdef CS_SCCP_REALTIME
@@ -122,6 +123,7 @@ sccp_device_t *sccp_device_find_realtime(const char *name);
 #endif
 #endif
 
-void sccp_device_setLamp(const sccp_device_t *device, skinny_stimulus_t stimulus, uint8_t instance, skinny_lampmode_t mode);
+void sccp_device_setLamp(const sccp_device_t * device, skinny_stimulus_t stimulus, uint8_t instance, skinny_lampmode_t mode);
 
 #endif														/* __SCCP_DEVICE_H */
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
