@@ -8,8 +8,8 @@
  *              Modified by Jan Czmok and Julien Goodwin
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2014-12-12 09:11:38 +0100 (Fri, 12 Dec 2014) $
+ * $Revision: 5810 $
  */
 
 #include <config.h>
@@ -22,7 +22,7 @@
 #include "sccp_conference.h"
 #include "sccp_actions.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5810 $")
 
 static void __sccp_indicate_remote_device(sccp_device_t * device, sccp_channel_t * c, sccp_line_t * line, sccp_channelstate_t state);
 
@@ -204,12 +204,12 @@ void __sccp_indicate(sccp_device_t * device, sccp_channel_t * c, sccp_channelsta
 			}
 
 			sccp_dev_set_keyset(d, instance, c->callid, KEYMODE_RINGIN);
-			/*                      char prompt[100];
-			   snprintf(prompt, sizeof(prompt), "%s%s: %s", (c->ringermode == SKINNY_RINGTYPE_URGENT) ? SKINNY_DISP_FLASH : "", strlen(c->callInfo.originalCalledPartyName) ? c->callInfo.originalCalledPartyName : SKINNY_DISP_FROM, strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
-			   sccp_dev_displayprompt(d, instance, c->callid, prompt, GLOB(digittimeout)); */
-			if (c->ringermode) {
-				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_FLASH, GLOB(digittimeout));
-			}
+			char prompt[100];
+			snprintf(prompt, sizeof(prompt), "%s%s: %s", (c->ringermode == SKINNY_RINGTYPE_URGENT) ? SKINNY_DISP_FLASH : "", strlen(c->callInfo.originalCalledPartyName) ? c->callInfo.originalCalledPartyName : SKINNY_DISP_FROM, strlen(c->callInfo.callingPartyName) ? c->callInfo.callingPartyName : c->callInfo.callingPartyNumber);
+			sccp_dev_displayprompt(d, instance, c->callid, prompt, GLOB(digittimeout));
+// 			if (c->ringermode) {
+// 				sccp_dev_displayprompt(d, instance, c->callid, SKINNY_DISP_FLASH, GLOB(digittimeout));
+// 			}
 
 			PBX(set_callstate) (c, AST_STATE_RINGING);						/*!\todo thats not the right place to update pbx state */
 			break;
