@@ -1,15 +1,15 @@
 /*!
- * \file	sccp_line.c
- * \brief	SCCP Line
- * \author	Sergio Chersovani <mlists [at] c-net.it>
- * \note	Reworked, but based on chan_sccp code.
- * 		The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
- * 		Modified by Jan Czmok and Julien Goodwin
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
+ * \file        sccp_line.c
+ * \brief       SCCP Line
+ * \author      Sergio Chersovani <mlists [at] c-net.it>
+ * \note        Reworked, but based on chan_sccp code.
+ *              The original chan_sccp driver that was made by Zozo which itself was derived from the chan_skinny driver.
+ *              Modified by Jan Czmok and Julien Goodwin
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-08-07 15:15:04 +0200 (Thu, 07 Aug 2014) $
- * $Revision: 5629 $
+ * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
+ * $Revision: 5790 $
  */
 
 #include <config.h>
@@ -22,7 +22,7 @@
 #include "sccp_config.h"
 #include "sccp_mwi.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5629 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
 static void regcontext_exten(sccp_line_t * l, struct subscriptionId *subscriptionId, int onoff);
 int __sccp_line_destroy(const void *ptr);
 int __sccp_lineDevice_destroy(const void *ptr);
@@ -84,6 +84,7 @@ void sccp_line_post_reload(void)
 			continue;
 		}
 		AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
+
 		if (l) {
 			SCCP_LIST_LOCK(&l->devices);
 			SCCP_LIST_TRAVERSE(&l->devices, linedevice, list) {
@@ -116,13 +117,13 @@ void sccp_line_post_reload(void)
 sccp_line_t *sccp_line_create(const char *name)
 {
 	sccp_line_t *l = NULL;
-	
-/* 	// do make sure line->name is unique before adding.
+
+	/*      // do make sure line->name is unique before adding.
 	if ((l = sccp_line_find_byname(name, FALSE))) {
-		sccp_line_release(l);
-		return NULL;
+	   sccp_line_release(l);
+	   return NULL;
 	}
-*/
+	*/
 	l = (sccp_line_t *) sccp_refcount_object_alloc(sizeof(sccp_line_t), SCCP_REF_LINE, name, __sccp_line_destroy);
 
 	if (!l) {
@@ -219,8 +220,9 @@ void *sccp_create_hotline(void)
 	}
 	memset(GLOB(hotline), 0, sizeof(sccp_hotline_t));
 
-//	SCCP_RWLIST_WRLOCK(&GLOB(lines));
+	//SCCP_RWLIST_WRLOCK(&GLOB(lines));
 	AUTO_RELEASE sccp_line_t *hotline = sccp_line_create("Hotline");
+
 	if (hotline) {
 #ifdef CS_SCCP_REALTIME
 		hotline->realtime = TRUE;
@@ -237,7 +239,7 @@ void *sccp_create_hotline(void)
 		GLOB(hotline)->line = sccp_line_retain(hotline);						// retain line inside hotline
 		sccp_line_addToGlobals(hotline);								// retain line inside GlobalsList
 	}
-//	SCCP_RWLIST_UNLOCK(&GLOB(lines));
+	//SCCP_RWLIST_UNLOCK(&GLOB(lines));
 
 	return NULL;
 }
@@ -260,6 +262,7 @@ void sccp_line_kill_channels(sccp_line_t * l)
 	// SCCP_LIST_LOCK(&l->channels);
 	SCCP_LIST_TRAVERSE_SAFE_BEGIN(&l->channels, c, list) {
 		AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
+
 		sccp_channel_endcall(channel);
 	}
 	SCCP_LIST_TRAVERSE_SAFE_END;
@@ -311,9 +314,9 @@ int __sccp_line_destroy(const void *ptr)
 		SCCP_LIST_HEAD_DESTROY(&l->devices);
 	}
 	// cleanup mailboxes
-//	if (l->trnsfvm) {
-//		sccp_free(l->trnsfvm);
-//	}
+	//if (l->trnsfvm) {
+	//  sccp_free(l->trnsfvm);
+	//}
 	sccp_mailbox_t *mailbox = NULL;
 
 	SCCP_LIST_LOCK(&l->mailboxes);
@@ -334,17 +337,16 @@ int __sccp_line_destroy(const void *ptr)
 	if (SCCP_LIST_EMPTY(&l->mailboxes)) {
 		SCCP_LIST_HEAD_DESTROY(&l->mailboxes);
 	}
-//#if CS_AST_HAS_NAMEDGROUP
-//	if (l->namedcallgroup) {
-//		sccp_free(l->namedcallgroup);
-//	}
-//	if (l->namedpickupgroup) {
-//		sccp_free(l->namedpickupgroup);
-//	}
-//#endif
+	//#if CS_AST_HAS_NAMEDGROUP
+	//if (l->namedcallgroup) {
+	//  sccp_free(l->namedcallgroup);
+	//}
+	//if (l->namedpickupgroup) {
+	//  sccp_free(l->namedpickupgroup);
+	//}
+	//#endif
 	sccp_config_cleanup_dynamically_allocated_memory(l, SCCP_CONFIG_LINE_SEGMENT);
-	
-	
+
 	// cleanup channels
 	sccp_channel_t *c = NULL;
 
@@ -430,6 +432,7 @@ void sccp_line_delete_nolock(sccp_line_t * l)
 void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward_t type, char *number)
 {
 	sccp_linedevices_t *linedevice = NULL;
+	sccp_feature_type_t feature_type = SCCP_FEATURE_CFWDNONE;
 
 	if (!line || !device) {
 		return;
@@ -443,14 +446,16 @@ void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward
 			if (!number || sccp_strlen_zero(number)) {
 				linedevice->cfwdAll.enabled = 0;
 				linedevice->cfwdBusy.enabled = 0;
-				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward to an empty number. Invalid\n", DEV_ID_LOG(device));
+				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward to an empty number. Invalid. Cfwd Disabled\n", DEV_ID_LOG(device));
 			} else {
 				switch (type) {
 					case SCCP_CFWD_ALL:
+						feature_type = SCCP_FEATURE_CFWDALL;
 						linedevice->cfwdAll.enabled = 1;
 						sccp_copy_string(linedevice->cfwdAll.number, number, sizeof(linedevice->cfwdAll.number));
 						break;
 					case SCCP_CFWD_BUSY:
+						feature_type = SCCP_FEATURE_CFWDBUSY;
 						linedevice->cfwdBusy.enabled = 1;
 						sccp_copy_string(linedevice->cfwdBusy.number, number, sizeof(linedevice->cfwdBusy.number));
 						break;
@@ -458,11 +463,11 @@ void sccp_line_cfwd(sccp_line_t * line, sccp_device_t * device, sccp_callforward
 						linedevice->cfwdAll.enabled = 0;
 						linedevice->cfwdBusy.enabled = 0;
 				}
-				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward enabled on line %s to number %s\n", DEV_ID_LOG(device), line->name, number);
+				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: Call Forward %s enabled on line %s to number %s\n", DEV_ID_LOG(device), sccp_callforward2str(type), line->name, number);
 			}
 		}
 		sccp_dev_starttone(linedevice->device, SKINNY_TONE_ZIPZIP, 0, 0, 0);
-		sccp_feat_changed(linedevice->device, linedevice, SCCP_FEATURE_CFWDALL);
+		sccp_feat_changed(linedevice->device, linedevice, feature_type);
 		sccp_dev_forward_status(linedevice->line, linedevice->lineInstance, device);
 		linedevice = sccp_linedevice_release(linedevice);
 	} else {
@@ -489,6 +494,7 @@ void sccp_line_addDevice(sccp_line_t * line, sccp_device_t * d, uint8_t lineInst
 	}
 
 	sccp_linedevices_t *linedevice = NULL;
+
 	if ((linedevice = sccp_linedevice_find(device, l))) {
 		sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_3 "%s: device already registered for line '%s'\n", DEV_ID_LOG(device), l->name);
 		sccp_linedevice_release(linedevice);
@@ -519,7 +525,7 @@ void sccp_line_addDevice(sccp_line_t * line, sccp_device_t * d, uint8_t lineInst
 
 	linedevice->line->statistic.numberOfActiveDevices++;
 	linedevice->device->configurationStatistic.numberOfLines++;
-	
+
 	// fire event for new device
 	sccp_event_t event;
 
@@ -592,6 +598,7 @@ void sccp_line_addChannel(sccp_line_t * line, sccp_channel_t * channel)
 	sccp_channel_t *c = NULL;
 
 	AUTO_RELEASE sccp_line_t *l = sccp_line_retain(line);
+
 	if (l) {
 		l->statistic.numberOfActiveChannels++;
 		SCCP_LIST_LOCK(&l->channels);
@@ -602,7 +609,7 @@ void sccp_line_addChannel(sccp_line_t * line, sccp_channel_t * channel)
 				SCCP_LIST_INSERT_TAIL(&l->channels, c, list);					// add to list
 			} else {
 				SCCP_LIST_INSERT_HEAD(&l->channels, c, list);					// add to list
-			}	
+			}
 		}
 		SCCP_LIST_UNLOCK(&l->channels);
 	}
@@ -631,7 +638,7 @@ void sccp_line_removeChannel(sccp_line_t * line, sccp_channel_t * channel)
 		if ((c = SCCP_LIST_REMOVE(&l->channels, channel, list))) {
 			sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_1 "SCCP: Removing channel %d from line %s\n", c->callid, l->name);
 			l->statistic.numberOfActiveChannels--;
-			channel = sccp_channel_release(c);						// Remove retain from list
+			channel = sccp_channel_release(c);							// Remove retain from list
 		}
 		SCCP_LIST_UNLOCK(&l->channels);
 	}
@@ -691,7 +698,7 @@ static void regcontext_exten(sccp_line_t * l, struct subscriptionId *subscriptio
 			} else {
 				/* un-register */
 
-				if (&l->devices && SCCP_LIST_GETSIZE(&l->devices) == 1) {							// only remove entry if it is the last one (shared line)
+				if (&l->devices && SCCP_LIST_GETSIZE(&l->devices) == 1) {			// only remove entry if it is the last one (shared line)
 					if (pbx_find_extension(NULL, NULL, &q, context, ext, 1, NULL, "", E_MATCH)) {
 						ast_context_remove_extension(context, ext, 1, NULL);
 						sccp_log((DEBUGCAT_LINE + DEBUGCAT_CONFIG)) (VERBOSE_PREFIX_1 "Unregistered RegContext: %s, Extension: %s\n", context, ext);
@@ -770,7 +777,7 @@ sccp_channelstate_t sccp_line_getDNDChannelState(sccp_line_t * line)
 sccp_line_t *sccp_line_find_byname(const char *name, uint8_t useRealtime)
 {
 	sccp_line_t *l = NULL;
-	
+
 	SCCP_RWLIST_RDLOCK(&GLOB(lines));
 	l = SCCP_RWLIST_FIND(&GLOB(lines), l, list, (sccp_strcaseequals(l->name, name)), TRUE);
 	SCCP_RWLIST_UNLOCK(&GLOB(lines));
@@ -830,16 +837,16 @@ sccp_line_t *sccp_line_find_realtime_byname(const char *name)
 
 		sccp_log((DEBUGCAT_LINE)) (VERBOSE_PREFIX_4 "SCCP: creating realtime line '%s'\n", name);
 
-//		SCCP_RWLIST_WRLOCK(&GLOB(lines));
-		if ((l = sccp_line_create(name))) {									/* already retained */
+		//  SCCP_RWLIST_WRLOCK(&GLOB(lines));
+		if ((l = sccp_line_create(name))) {								/* already retained */
 			sccp_config_applyLineConfiguration(l, variable);
 			l->realtime = TRUE;
-			sccp_line_addToGlobals(l);									// can return previous instance on doubles
+			sccp_line_addToGlobals(l);								// can return previous instance on doubles
 			pbx_variables_destroy(v);
 		} else {
 			pbx_log(LOG_ERROR, "SCCP: Unable to build realtime line '%s'\n", name);
 		}
-//		SCCP_RWLIST_UNLOCK(&GLOB(lines));
+		//  SCCP_RWLIST_UNLOCK(&GLOB(lines));
 		return l;
 	}
 
@@ -877,7 +884,6 @@ sccp_line_t *sccp_line_find_byid(sccp_device_t * d, uint16_t instance)
 #endif
 {
 	sccp_line_t *l = NULL;
-	
 
 	sccp_log((DEBUGCAT_LINE + DEBUGCAT_DEVICE)) (VERBOSE_PREFIX_3 "%s: Looking for line with instance %d.\n", DEV_ID_LOG(d), instance);
 
@@ -885,11 +891,11 @@ sccp_line_t *sccp_line_find_byid(sccp_device_t * d, uint16_t instance)
 		return NULL;
 	}
 
-	if (0 < instance && instance < d->lineButtons.size && d->lineButtons.instance[instance] && d->lineButtons.instance[instance]->line ){
+	if (0 < instance && instance < d->lineButtons.size && d->lineButtons.instance[instance] && d->lineButtons.instance[instance]->line) {
 #if DEBUG
-		l = sccp_refcount_retain( d->lineButtons.instance[instance]->line, filename, lineno, func);
+		l = sccp_refcount_retain(d->lineButtons.instance[instance]->line, filename, lineno, func);
 #else
-		l = sccp_line_retain( d->lineButtons.instance[instance]->line );
+		l = sccp_line_retain(d->lineButtons.instance[instance]->line);
 #endif
 	}
 
@@ -921,7 +927,8 @@ sccp_line_t *sccp_line_find_byid(sccp_device_t * d, uint16_t instance)
 sccp_linedevices_t *__sccp_linedevice_find(const sccp_device_t * device, const sccp_line_t * line, const char *filename, int lineno, const char *func)
 {
 	sccp_linedevices_t *linedevice = NULL;
-	sccp_line_t *l = (sccp_line_t *) line;       // loose const qualifier, to be able to lock the list;
+	sccp_line_t *l = (sccp_line_t *) line;									// loose const qualifier, to be able to lock the list;
+
 	if (!l) {
 		pbx_log(LOG_NOTICE, "%s: [%s:%d]->linedevice_find: No line provided to search in\n", DEV_ID_LOG(device), filename, lineno);
 		return NULL;
@@ -930,7 +937,7 @@ sccp_linedevices_t *__sccp_linedevice_find(const sccp_device_t * device, const s
 		pbx_log(LOG_NOTICE, "SCCP: [%s:%d]->linedevice_find: No device provided to search for (line: %s)\n", filename, lineno, line ? line->name : "UNDEF");
 		return NULL;
 	}
-	
+
 	SCCP_LIST_LOCK(&l->devices);
 	linedevice = SCCP_LIST_FIND(&l->devices, linedevice, list, (device == linedevice->device), TRUE);
 	SCCP_LIST_UNLOCK(&l->devices);
@@ -944,7 +951,7 @@ sccp_linedevices_t *__sccp_linedevice_find(const sccp_device_t * device, const s
 sccp_linedevices_t *__sccp_linedevice_findByLineinstance(const sccp_device_t * device, uint16_t instance, const char *filename, int lineno, const char *func)
 {
 	sccp_linedevices_t *linedevice = NULL;
-  
+
 	if (instance < 1) {
 		pbx_log(LOG_NOTICE, "%s: [%s:%d]->linedevice_find: No line provided to search in\n", DEV_ID_LOG(device), filename, lineno);
 		return NULL;
@@ -953,9 +960,9 @@ sccp_linedevices_t *__sccp_linedevice_findByLineinstance(const sccp_device_t * d
 		pbx_log(LOG_NOTICE, "SCCP: [%s:%d]->linedevice_find: No device provided to search for (lineinstance: %d)\n", filename, lineno, instance);
 		return NULL;
 	}
-	
-	if (0 < instance && instance < device->lineButtons.size && device->lineButtons.instance[instance]) {		/* 0 < instance < lineButton.size */
-		linedevice = sccp_linedevice_retain( device->lineButtons.instance[ instance ] ) ;
+
+	if (0 < instance && instance < device->lineButtons.size && device->lineButtons.instance[instance]) {	/* 0 < instance < lineButton.size */
+		linedevice = sccp_linedevice_retain(device->lineButtons.instance[instance]);
 	}
 
 	if (!linedevice) {
@@ -964,44 +971,44 @@ sccp_linedevices_t *__sccp_linedevice_findByLineinstance(const sccp_device_t * d
 	return linedevice;
 }
 
-
-
 /* create linebutton array */
-void sccp_line_createLineButtonsArray(sccp_device_t *device) {
+void sccp_line_createLineButtonsArray(sccp_device_t * device)
+{
 	sccp_linedevices_t *linedevice;
 	uint8_t lineInstances = 0;
 	btnlist *btn;
 	uint8_t i;
-	
-	if(device->lineButtons.instance){
+
+	if (device->lineButtons.instance) {
 		sccp_line_deleteLineButtonsArray(device);
 	}
-	
+
 	btn = device->buttonTemplate;
-	
+
 	for (i = 0; i < StationMaxButtonTemplateSize; i++) {
-		if (btn[i].type == SKINNY_BUTTONTYPE_LINE && btn[i].instance > lineInstances &&  btn[i].ptr) {
+		if (btn[i].type == SKINNY_BUTTONTYPE_LINE && btn[i].instance > lineInstances && btn[i].ptr) {
 			lineInstances = btn[i].instance;
-		} 
+		}
 	}
-	
+
 	device->lineButtons.size = lineInstances + SCCP_FIRST_LINEINSTANCE;					/* add the offset of SCCP_FIRST_LINEINSTANCE for explicit access */
-	device->lineButtons.instance = sccp_calloc(device->lineButtons.size, sizeof(sccp_linedevices_t *) );
-	
+	device->lineButtons.instance = sccp_calloc(device->lineButtons.size, sizeof(sccp_linedevices_t *));
+
 	for (i = 0; i < StationMaxButtonTemplateSize; i++) {
-		if (btn[i].type == SKINNY_BUTTONTYPE_LINE  && btn[i].ptr ) {
-			linedevice = sccp_linedevice_find(device, (sccp_line_t *)btn[i].ptr );
-			device->lineButtons.instance[ btn[i].instance ] = linedevice;
-		} 
+		if (btn[i].type == SKINNY_BUTTONTYPE_LINE && btn[i].ptr) {
+			linedevice = sccp_linedevice_find(device, (sccp_line_t *) btn[i].ptr);
+			device->lineButtons.instance[btn[i].instance] = linedevice;
+		}
 	}
 }
 
-void sccp_line_deleteLineButtonsArray(sccp_device_t *device) {
+void sccp_line_deleteLineButtonsArray(sccp_device_t * device)
+{
 	uint8_t i;
-  
-	if(device->lineButtons.instance){
+
+	if (device->lineButtons.instance) {
 		for (i = SCCP_FIRST_LINEINSTANCE; i < device->lineButtons.size; i++) {
-			if(device->lineButtons.instance[i]){ 
+			if (device->lineButtons.instance[i]) {
 				device->lineButtons.instance[i] = sccp_linedevice_release(device->lineButtons.instance[i]);
 			}
 		}
@@ -1009,3 +1016,5 @@ void sccp_line_deleteLineButtonsArray(sccp_device_t *device) {
 		sccp_free(device->lineButtons.instance);
 	}
 }
+
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
