@@ -8,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2014-12-13 18:49:44 +0100 (Sat, 13 Dec 2014) $
+ * $Revision: 5812 $
  */
 #ifndef __PBX_IMPL_C
 #define __PBX_IMPL_C
@@ -26,7 +26,7 @@
 #include "sccp_indicate.h"
 #include "sccp_socket.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5812 $")
 
     /*!
      * \brief SCCP Structure to pass data to the pbx answer thread 
@@ -232,6 +232,15 @@ int sccp_pbx_call(sccp_channel_t * c, char *dest, int timeout)
 
 		if (active_channel) {
 			sccp_indicate(linedevice->device, c, SCCP_CHANNELSTATE_CALLWAITING);
+			
+			/* display the new call on prompt */
+			{
+				AUTO_RELEASE sccp_linedevices_t *activeChannelLinedevice = sccp_linedevice_find(linedevice->device, active_channel->line);
+				
+				char prompt[100];
+				snprintf(prompt, sizeof(prompt), "%s: %s: %s", active_channel->line->name, SKINNY_DISP_FROM, c->callInfo.callingPartyNumber);
+				sccp_dev_displayprompt(linedevice->device, activeChannelLinedevice->lineInstance, active_channel->callid, prompt, GLOB(digittimeout));
+			}
 			isRinging = TRUE;
 		} else {
 
