@@ -8,8 +8,8 @@
  * \note                This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
+ * $Date: 2014-11-14 15:51:45 +0100 (Fri, 14 Nov 2014) $
+ * $Revision: 5798 $
  */
 
 #include <netinet/in.h>
@@ -19,7 +19,7 @@
 #include "sccp_device.h"
 #include "sccp_utils.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5798 $")
 #ifndef CS_USE_POLL_COMPAT
 #include <poll.h>
 #include <sys/poll.h>
@@ -1039,13 +1039,6 @@ void sccp_session_sendmsg(const sccp_device_t * device, sccp_mid_t t)
 
 	sccp_msg_t *msg = sccp_build_packet(t, 0);
 
-	if (msg && (GLOB(debug) & DEBUGCAT_MESSAGE) != 0) {
-		uint32_t mid = letohl(msg->header.lel_messageId);
-
-		pbx_log(LOG_NOTICE, "%s: Send Message: %s(0x%04X) %d bytes length\n", DEV_ID_LOG(device), msgtype2str(mid), mid, msg ? msg->header.length : 0);
-		sccp_dump_msg(msg);
-	}
-
 	if (msg) {
 		sccp_session_send(device, msg);
 	}
@@ -1108,6 +1101,13 @@ int sccp_session_send2(sccp_session_t * s, sccp_msg_t * msg)
 		msg->header.lel_protocolVer = 0;
 	}
 
+	if (msg && (GLOB(debug) & DEBUGCAT_MESSAGE) != 0) {
+		uint32_t mid = letohl(msg->header.lel_messageId);
+
+		pbx_log(LOG_NOTICE, "%s: Send Message: %s(0x%04X) %d bytes length\n", DEV_ID_LOG(s->device), msgtype2str(mid), mid, msg ? msg->header.length : 0);
+		sccp_dump_msg(msg);
+	}
+
 	try = 0;
 	maxTries = 500;												/* arbitrairy number of tries */
 	bytesSent = 0;
