@@ -8,8 +8,8 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2014-11-20 16:14:19 +0100 (Thu, 20 Nov 2014) $
- * $Revision: 5804 $
+ * $Date: 2014-11-27 14:34:55 +0100 (Thu, 27 Nov 2014) $
+ * $Revision: 5809 $
  */
 
 #include <config.h>
@@ -25,7 +25,7 @@
 #include "sccp_actions.h"
 #include "sccp_rtp.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5804 $")
+SCCP_FILE_VERSION(__FILE__, "$Revision: 5809 $")
 
     /* =========================================================================================== Global */
     /*!
@@ -405,26 +405,30 @@ static void sccp_sk_dirtrfr(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_dev
 {
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Direct Transfer Pressed\n", DEV_ID_LOG(d));
 
-	sccp_device_t *device = sccp_device_retain(d);
+	AUTO_RELEASE sccp_device_t *device = sccp_device_retain(d);
 
 	if (!device) {
 		return;
 	}
 
-	sccp_selectedchannel_t *x;
-	sccp_channel_t *chan1 = NULL, *chan2 = NULL, *tmp = NULL;
+	AUTO_RELEASE sccp_channel_t *chan1 = NULL, *chan2 = NULL;
+	sccp_channel_t *tmp = NULL;
 
-	if ((sccp_device_selectedchannels_count(device)) != 2) {
+	if ((sccp_device_selectedchannels_count(device)) == 2) {
+		sccp_selectedchannel_t *x = NULL;
+		SCCP_LIST_LOCK(&device->selectedChannels);
+		x = SCCP_LIST_FIRST(&device->selectedChannels);
+		chan1 = sccp_channel_retain(x->channel);
+		chan2 = SCCP_LIST_NEXT(x, list)->channel;
+		chan2 = sccp_channel_retain(chan2);
+		SCCP_LIST_UNLOCK(&device->selectedChannels);
+	} else {
 		if (SCCP_RWLIST_GETSIZE(&l->channels) == 2) {
-			sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: Automatically select the two current channels\n", device->id);
 			SCCP_LIST_LOCK(&l->channels);
-			SCCP_LIST_TRAVERSE(&l->channels, c, list) {
-				x = sccp_malloc(sizeof(sccp_selectedchannel_t));
-				if (x != NULL) {
-					x->channel = c;
-					SCCP_LIST_LOCK(&device->selectedChannels);
-					SCCP_LIST_INSERT_HEAD(&device->selectedChannels, x, list);
-					SCCP_LIST_UNLOCK(&device->selectedChannels);
+			if ((tmp  = SCCP_LIST_FIRST(&l->channels))) {
+				chan1 = sccp_channel_retain(tmp);
+				if ((tmp = SCCP_LIST_NEXT(tmp, list))) {
+					chan2 = sccp_channel_retain(tmp);
 				}
 			}
 			SCCP_LIST_UNLOCK(&l->channels);
@@ -439,12 +443,6 @@ static void sccp_sk_dirtrfr(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_dev
 		}
 	}
 
-	SCCP_LIST_LOCK(&device->selectedChannels);
-	x = SCCP_LIST_FIRST(&device->selectedChannels);
-	chan1 = sccp_channel_retain(x->channel);
-	chan2 = SCCP_LIST_NEXT(x, list)->channel;
-	chan2 = sccp_channel_retain(chan2);
-	SCCP_LIST_UNLOCK(&device->selectedChannels);
 
 	if (chan1 && chan2) {
 		//for using the sccp_channel_transfer_complete function
@@ -466,10 +464,7 @@ static void sccp_sk_dirtrfr(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_dev
 		device->transferChannels.transferer = sccp_channel_retain(chan2);
 
 		sccp_channel_transfer_complete(chan2);
-		chan1 = sccp_channel_release(chan1);
-		chan2 = sccp_channel_release(chan2);
 	}
-	device = sccp_device_release(device);
 }
 
 /*!
@@ -725,7 +720,12 @@ static void sccp_sk_conference(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_
 static void sccp_sk_conflist(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance, sccp_channel_t * c)
 {
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Conflist Pressed\n", DEV_ID_LOG(d));
+#ifdef CS_SCCP_CONFERENCE
 	sccp_feat_conflist(d, l, lineInstance, c);
+#else
+	sccp_dev_displayprompt(d, lineInstance, c->callid, SKINNY_DISP_KEY_IS_NOT_ACTIVE, SCCP_DISPLAYSTATUS_TIMEOUT);
+	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "### Conference was not compiled in\n");
+#endif
 }
 
 /*!
@@ -736,7 +736,12 @@ static void sccp_sk_conflist(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_de
 static void sccp_sk_join(const sccp_softkeyMap_cb_t * softkeyMap_cb, sccp_device_t * d, sccp_line_t * l, const uint32_t lineInstance, sccp_channel_t * c)
 {
 	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "%s: SoftKey Join Pressed\n", DEV_ID_LOG(d));
+#ifdef CS_SCCP_CONFERENCE
 	sccp_feat_join(d, l, lineInstance, c);
+#else
+	sccp_dev_displayprompt(d, lineInstance, c->callid, SKINNY_DISP_KEY_IS_NOT_ACTIVE, SCCP_DISPLAYSTATUS_TIMEOUT);
+	sccp_log((DEBUGCAT_SOFTKEY)) (VERBOSE_PREFIX_3 "### Conference was not compiled in\n");
+#endif
 }
 
 /*!
@@ -957,18 +962,12 @@ static const struct sccp_softkeyMap_cb softkeyCbMap[] = {
 	{SKINNY_LBL_INTRCPT, sccp_sk_resume, TRUE, NULL},
 	{SKINNY_LBL_DIAL, sccp_sk_dial, TRUE, NULL},
 	{SKINNY_LBL_VIDEO_MODE, sccp_sk_videomode, TRUE, NULL},
-#ifdef CS_SCCP_PARK
 	{SKINNY_LBL_PARK, sccp_sk_park, TRUE, NULL},
-#endif
-#ifdef CS_SCCP_PICKUP
 	{SKINNY_LBL_PICKUP, sccp_sk_pickup, FALSE, NULL},
 	{SKINNY_LBL_GPICKUP, sccp_sk_gpickup, FALSE, NULL},
-#endif
-#ifdef CS_SCCP_CONFERENCE
 	{SKINNY_LBL_CONFRN, sccp_sk_conference, TRUE, NULL},
 	{SKINNY_LBL_JOIN, sccp_sk_join, TRUE, NULL},
 	{SKINNY_LBL_CONFLIST, sccp_sk_conflist, TRUE, NULL},
-#endif
 };
 
 /*!
